#ifndef PROTOCOL_H
#define PROTOCOL_H

/*
 * q931 hat mich beeinflusst ;)
 *
 * octet 1
 * TAAATSSS	T = typ, A = acknowledge, S = sequence
 *
 */

#define PROTOCOL_VERSION 3

#define MAX_PACKET	100	// gross over-estimate suffices

#define MSG_MASK	0x11

enum {	MSG_ACK		= 0x11,	// ack-message, only ack value is valid
	MSG_MIXED	= 0x10,	// controllbytes + daten
	MSG_DATA	= 0x01,	// daten
	MSG_SYN		= 0x00,	// init
	MSG_DISCOVERY	= 0x02, // MSG_DISCOVERY src_mask dst_mask (discovery protocol)
	MSG_CFG		= 0x04, // config-protocol messages
	MSG_RST		= 0x06, // stop all transfers immediately
     };

// itcp

#define WINDOW		3	// max. number of unacknowledged packets

#define SEQ_MASK	7	// three bits..
#define MSG_SEQ(head)	(((head) >> 1) & SEQ_MASK)
#define MSG_ACK(head)	(((head) >> 5) & SEQ_MASK)
#define MSG_HEAD(type, ack, seq) \
			((type) | ( ((ack) & SEQ_MASK) << 5) | (((seq) & SEQ_MASK) << 1))

/* itcp-kontrollbytes */

#define CTL_MASK	0xf8

/* obere 5 bits */
#define CTL_ATOM	0x80	// ohne parameter
#define CTL_RESEND	0x90	// resend packet with sequence in the low three bits
#define CTL_CMD		0xa0	// mit 1-byte parameter

#define CTL_ATOM_NOP	0x80	// fllbyte
#define CTL_ATOM_PING	0x81	// hin
#define CTL_ATOM_PONG	0x82	// zurck
#define CTL_ATOM_FIN	0x83	// ende der verbindung

#define CTL_ATOM_END	0x87	// ende der kontrollbytes

#define CTL_CMD_LACK	0xa0	// large acknowledgment in next byte
#define CTL_CMD_FLAGS	0xa1	// set pflags for this connection (bidirectional)

// gerneral config protocol (yeah, get's complicated)

// replies have bit 7 set
#define CFG_REPLY	0x80
#define CFG_POLL	0x01
#define CFG_MSG		0x02

// standard port
#define PORT_UNSPEC	0
#define PORT_MAXUSER	64
#define PORT_CHAT	65
#define PORT_ITCP	66	// default itcp port
#define PORT_RESERVED	226
#define PORT_MAX	255

// protocol flags
#define PFLAG_GZIP	0x01	// request gzip
#define PFLAG_LACK	0x02	// large acknowledgment supported
#define PFLAG_CAN	(PFLAG_GZIP)
#define PFLAG_WANT	(PFLAG_GZIP)

/*

  MSG_DISCOVERY	src_mask dst_mask	// MUST NOT be longer
  MSG_SYN	protocolversion
  		src_port dst_port
  		protflags		// options we support
  MSG_RST	cause			// cause is zero
  
  MSG_CFG
    CFG_POLL	01 protocolversion #connections servername // poll server configuration
    CFG_MSG	message		// display message

*/

#endif

